@@ -1,22 +1,23 @@
 use strict; use warnings;
 
-use Object::Pad;
+use Feature::Compat::Class;
 use CAD::OpenSCAD::Math;
 
 =pod
 
 =head1 NAME
 
-CAD::OpenSCAD::Gearmaker - A module generate gears for CAD::OpenSCAD
+CAD::OpenSCAD::Gearmaker - A module to generate gears for CAD::OpenSCAD
 
 =cut
 
-our $VERSION='0.15';
+our $VERSION='0.16';
 	
 our $Math=new CAD::OpenSCAD::Math;
 		
 class CAD::OpenSCAD::GearMaker{
 	field $scad :param;	
+	our $VERSION='0.16';
 	
 	method profile{
 		my %params=@_;
